@@ -210,6 +210,9 @@ PYCAIRO_MODINIT_FUNC PyInit__cairo(void)
   if (PyType_Ready(&PycairoTextExtents_Type) < 0)
     return NULL;
 
+  if (PyType_Ready(&PycairoSurface_Type) < 0)
+    return NULL;
+
 #ifdef CAIRO_HAS_SCRIPT_SURFACE
   if (PyType_Ready(&PycairoScriptDevice_Type) < 0)
     return NULL;
@@ -223,8 +226,6 @@ PYCAIRO_MODINIT_FUNC PyInit__cairo(void)
   if (PyType_Ready(&PycairoScaledFont_Type) < 0)
     return NULL;
 
-  if (PyType_Ready(&PycairoSurface_Type) < 0)
-    return NULL;
 #ifdef CAIRO_HAS_IMAGE_SURFACE
   if (PyType_Ready(&PycairoImageSurface_Type) < 0)
     return NULL;
@@ -379,6 +380,10 @@ PYCAIRO_MODINIT_FUNC PyInit__cairo(void)
 #endif
 #endif
 
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 17, 8)
+  PyModule_AddIntConstant(m, "COLOR_PALETTE_DEFAULT", CAIRO_COLOR_PALETTE_DEFAULT);
+#endif
+
 #ifdef CAIRO_HAS_PS_SURFACE
   Py_INCREF(&PycairoPSSurface_Type);
   PyModule_AddObject(m, "PSSurface", (PyObject *)&PycairoPSSurface_Type);
@@ -543,6 +548,10 @@ PYCAIRO_MODINIT_FUNC PyInit__cairo(void)
   STRCONSTANT(TAG_DEST);
   STRCONSTANT(TAG_LINK);
 #endif
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 18, 0)
+  STRCONSTANT(TAG_CONTENT);
+  STRCONSTANT(TAG_CONTENT_REF);
+#endif
 
 #undef STRCONSTANT
 
