@@ -74,6 +74,69 @@ pycairo_new (PyTypeObject *type, PyObject *args, PyObject *kwds) {
   return PycairoContext_FromContext (cairo_create (s->surface), type, NULL);
 }
 
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 17, 6)
+static PyObject *
+pycairo_set_hairline (PycairoContext *o, PyObject *args) {
+    PyObject *py_hairline;
+    if (!PyArg_ParseTuple(args, "O!:Context.set_hairline",
+                &PyBool_Type, &py_hairline))
+        return NULL;
+    cairo_set_hairline (o->ctx, (py_hairline == Py_True));
+    RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+pycairo_get_hairline (PycairoContext *o, PyObject *ignored) {
+  PyObject *set_hairline = cairo_get_hairline (o->ctx) ? Py_True : Py_False;
+  RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
+  Py_INCREF(set_hairline);
+  return set_hairline;
+}
+#endif
+
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 15, 10)
+static PyObject *
+pycairo_tag_begin (PycairoContext *o, PyObject *args) {
+    const char *tag_name;
+    const char *attributes;
+
+    if (!PyArg_ParseTuple (args,
+            "eses:Context.tag_begin", "utf-8",
+            &tag_name, "utf-8", &attributes))
+        return NULL;
+
+    Py_BEGIN_ALLOW_THREADS;
+    cairo_tag_begin (o->ctx, tag_name, attributes);
+    Py_END_ALLOW_THREADS;
+
+    PyMem_Free((void *)tag_name);
+    PyMem_Free((void *)attributes);
+
+    RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+pycairo_tag_end (PycairoContext *o, PyObject *args) {
+    const char *tag_name;
+
+    if (!PyArg_ParseTuple (args,
+            "es:Context.tag_end", "utf-8",
+            &tag_name))
+        return NULL;
+
+      Py_BEGIN_ALLOW_THREADS;
+      cairo_tag_end (o->ctx, tag_name);
+      Py_END_ALLOW_THREADS;
+
+      PyMem_Free((void *)tag_name);
+
+      RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
+      Py_RETURN_NONE;
+}
+#endif
+
 static PyObject *
 pycairo_append_path (PycairoContext *o, PyObject *args) {
   PycairoPath *p;
@@ -116,7 +179,7 @@ pycairo_arc_negative (PycairoContext *o, PyObject *args) {
 }
 
 static PyObject *
-pycairo_clip (PycairoContext *o) {
+pycairo_clip (PycairoContext *o, PyObject *ignored) {
   Py_BEGIN_ALLOW_THREADS;
   cairo_clip (o->ctx);
   Py_END_ALLOW_THREADS;
@@ -125,7 +188,7 @@ pycairo_clip (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_clip_extents (PycairoContext *o) {
+pycairo_clip_extents (PycairoContext *o, PyObject *ignored) {
   double x1, y1, x2, y2;
   cairo_clip_extents (o->ctx, &x1, &y1, &x2, &y2);
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
@@ -133,7 +196,7 @@ pycairo_clip_extents (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_clip_preserve (PycairoContext *o) {
+pycairo_clip_preserve (PycairoContext *o, PyObject *ignored) {
   Py_BEGIN_ALLOW_THREADS;
   cairo_clip_preserve (o->ctx);
   Py_END_ALLOW_THREADS;
@@ -157,7 +220,7 @@ pycairo_in_clip (PycairoContext *o, PyObject *args) {
 }
 
 static PyObject *
-pycairo_close_path (PycairoContext *o) {
+pycairo_close_path (PycairoContext *o, PyObject *ignored) {
   Py_BEGIN_ALLOW_THREADS;
   cairo_close_path (o->ctx);
   Py_END_ALLOW_THREADS;
@@ -166,7 +229,7 @@ pycairo_close_path (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_copy_clip_rectangle_list (PycairoContext *o) {
+pycairo_copy_clip_rectangle_list (PycairoContext *o, PyObject *ignored) {
   int i;
   PyObject *rv = NULL;
   PyObject *rect = NULL;
@@ -203,7 +266,7 @@ pycairo_copy_clip_rectangle_list (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_copy_page (PycairoContext *o) {
+pycairo_copy_page (PycairoContext *o, PyObject *ignored) {
   Py_BEGIN_ALLOW_THREADS;
   cairo_copy_page (o->ctx);
   Py_END_ALLOW_THREADS;
@@ -212,7 +275,7 @@ pycairo_copy_page (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_copy_path (PycairoContext *o) {
+pycairo_copy_path (PycairoContext *o, PyObject *ignored) {
   cairo_path_t *cp;
   Py_BEGIN_ALLOW_THREADS;
   cp = cairo_copy_path (o->ctx);
@@ -221,7 +284,7 @@ pycairo_copy_path (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_copy_path_flat (PycairoContext *o) {
+pycairo_copy_path_flat (PycairoContext *o, PyObject *ignored) {
   cairo_path_t *cp;
   Py_BEGIN_ALLOW_THREADS;
   cp = cairo_copy_path_flat (o->ctx);
@@ -268,7 +331,7 @@ pycairo_device_to_user_distance (PycairoContext *o, PyObject *args) {
 }
 
 static PyObject *
-pycairo_fill (PycairoContext *o) {
+pycairo_fill (PycairoContext *o, PyObject *ignored) {
   Py_BEGIN_ALLOW_THREADS;
   cairo_fill (o->ctx);
   Py_END_ALLOW_THREADS;
@@ -277,7 +340,7 @@ pycairo_fill (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_fill_extents (PycairoContext *o) {
+pycairo_fill_extents (PycairoContext *o, PyObject *ignored) {
   double x1, y1, x2, y2;
   cairo_fill_extents (o->ctx, &x1, &y1, &x2, &y2);
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
@@ -285,7 +348,7 @@ pycairo_fill_extents (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_fill_preserve (PycairoContext *o) {
+pycairo_fill_preserve (PycairoContext *o, PyObject *ignored) {
   Py_BEGIN_ALLOW_THREADS;
   cairo_fill_preserve (o->ctx);
   Py_END_ALLOW_THREADS;
@@ -294,7 +357,7 @@ pycairo_fill_preserve (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_font_extents (PycairoContext *o) {
+pycairo_font_extents (PycairoContext *o, PyObject *ignored) {
   cairo_font_extents_t e;
 
   cairo_font_extents (o->ctx, &e);
@@ -304,19 +367,19 @@ pycairo_font_extents (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_get_antialias (PycairoContext *o) {
+pycairo_get_antialias (PycairoContext *o, PyObject *ignored) {
   RETURN_INT_ENUM (Antialias, cairo_get_antialias (o->ctx));
 }
 
 static PyObject *
-pycairo_get_current_point (PycairoContext *o) {
+pycairo_get_current_point (PycairoContext *o, PyObject *ignored) {
   double x, y;
   cairo_get_current_point (o->ctx, &x, &y);
   return Py_BuildValue("(dd)", x, y);
 }
 
 static PyObject *
-pycairo_get_dash (PycairoContext *o) {
+pycairo_get_dash (PycairoContext *o, PyObject *ignored) {
   double *dashes = NULL, offset;
   int count, i;
   PyObject *py_dashes = NULL, *rv = NULL;
@@ -350,30 +413,30 @@ pycairo_get_dash (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_get_dash_count (PycairoContext *o) {
-  return PYCAIRO_PyLong_FromLong (cairo_get_dash_count (o->ctx));
+pycairo_get_dash_count (PycairoContext *o, PyObject *ignored) {
+  return PyLong_FromLong (cairo_get_dash_count (o->ctx));
 }
 
 static PyObject *
-pycairo_get_fill_rule (PycairoContext *o) {
+pycairo_get_fill_rule (PycairoContext *o, PyObject *ignored) {
   RETURN_INT_ENUM(FillRule, cairo_get_fill_rule (o->ctx));
 }
 
 static PyObject *
-pycairo_get_font_face (PycairoContext *o) {
+pycairo_get_font_face (PycairoContext *o, PyObject *ignored) {
   return PycairoFontFace_FromFontFace (
        cairo_font_face_reference (cairo_get_font_face (o->ctx)));
 }
 
 static PyObject *
-pycairo_get_font_matrix (PycairoContext *o) {
+pycairo_get_font_matrix (PycairoContext *o, PyObject *ignored) {
   cairo_matrix_t matrix;
   cairo_get_font_matrix (o->ctx, &matrix);
   return PycairoMatrix_FromMatrix (&matrix);
 }
 
 static PyObject *
-pycairo_get_font_options (PycairoContext *o) {
+pycairo_get_font_options (PycairoContext *o, PyObject *ignored) {
   cairo_font_options_t *options = cairo_font_options_create();
   cairo_get_font_options (o->ctx, options);
   /* there is no reference fn */
@@ -381,64 +444,64 @@ pycairo_get_font_options (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_get_group_target (PycairoContext *o) {
+pycairo_get_group_target (PycairoContext *o, PyObject *ignored) {
   cairo_surface_t *surface = cairo_get_group_target (o->ctx);
   return PycairoSurface_FromSurface (cairo_surface_reference (surface), NULL);
 }
 
 static PyObject *
-pycairo_get_line_cap (PycairoContext *o) {
+pycairo_get_line_cap (PycairoContext *o, PyObject *ignored) {
   RETURN_INT_ENUM(LineCap, cairo_get_line_cap (o->ctx));
 }
 
 static PyObject *
-pycairo_get_line_join (PycairoContext *o) {
+pycairo_get_line_join (PycairoContext *o, PyObject *ignored) {
   RETURN_INT_ENUM(LineJoin, cairo_get_line_join (o->ctx));
 }
 
 static PyObject *
-pycairo_get_line_width (PycairoContext *o) {
+pycairo_get_line_width (PycairoContext *o, PyObject *ignored) {
   return PyFloat_FromDouble(cairo_get_line_width (o->ctx));
 }
 
 static PyObject *
-pycairo_get_matrix (PycairoContext *o) {
+pycairo_get_matrix (PycairoContext *o, PyObject *ignored) {
   cairo_matrix_t matrix;
   cairo_get_matrix (o->ctx, &matrix);
   return PycairoMatrix_FromMatrix (&matrix);
 }
 
 static PyObject *
-pycairo_get_miter_limit (PycairoContext *o) {
+pycairo_get_miter_limit (PycairoContext *o, PyObject *ignored) {
   return PyFloat_FromDouble (cairo_get_miter_limit (o->ctx));
 }
 
 static PyObject *
-pycairo_get_operator (PycairoContext *o) {
+pycairo_get_operator (PycairoContext *o, PyObject *ignored) {
   RETURN_INT_ENUM(Operator, cairo_get_operator (o->ctx));
 }
 
 static PyObject *
-pycairo_get_scaled_font (PycairoContext *o) {
+pycairo_get_scaled_font (PycairoContext *o, PyObject *ignored) {
   return PycairoScaledFont_FromScaledFont (
 	   cairo_scaled_font_reference (cairo_get_scaled_font (o->ctx)));
 }
 
 static PyObject *
-pycairo_get_source (PycairoContext *o) {
+pycairo_get_source (PycairoContext *o, PyObject *ignored) {
   return PycairoPattern_FromPattern (
 	       cairo_pattern_reference (cairo_get_source (o->ctx)), NULL);
 }
 
 static PyObject *
-pycairo_get_target (PycairoContext *o) {
+pycairo_get_target (PycairoContext *o, PyObject *ignored) {
   return PycairoSurface_FromSurface (
 	       cairo_surface_reference (cairo_get_target (o->ctx)),
 	       NULL);
 }
 
 static PyObject *
-pycairo_get_tolerance (PycairoContext *o) {
+pycairo_get_tolerance (PycairoContext *o, PyObject *ignored) {
   return PyFloat_FromDouble (cairo_get_tolerance (o->ctx));
 }
 
@@ -487,7 +550,7 @@ pycairo_glyph_path (PycairoContext *o, PyObject *args) {
 }
 
 static PyObject *
-pycairo_has_current_point (PycairoContext *o) {
+pycairo_has_current_point (PycairoContext *o, PyObject *ignored) {
   PyObject *b = cairo_has_current_point (o->ctx) ? Py_True : Py_False;
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
   Py_INCREF(b);
@@ -495,7 +558,7 @@ pycairo_has_current_point (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_identity_matrix (PycairoContext *o) {
+pycairo_identity_matrix (PycairoContext *o, PyObject *ignored) {
   cairo_identity_matrix (o->ctx);
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
   Py_RETURN_NONE;
@@ -584,21 +647,21 @@ pycairo_move_to (PycairoContext *o, PyObject *args) {
 }
 
 static PyObject *
-pycairo_new_path (PycairoContext *o) {
+pycairo_new_path (PycairoContext *o, PyObject *ignored) {
   cairo_new_path (o->ctx);
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
   Py_RETURN_NONE;
 }
 
 static PyObject *
-pycairo_new_sub_path (PycairoContext *o) {
+pycairo_new_sub_path (PycairoContext *o, PyObject *ignored) {
   cairo_new_sub_path (o->ctx);
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
   Py_RETURN_NONE;
 }
 
 static PyObject *
-pycairo_paint (PycairoContext *o) {
+pycairo_paint (PycairoContext *o, PyObject *ignored) {
   Py_BEGIN_ALLOW_THREADS;
   cairo_paint (o->ctx);
   Py_END_ALLOW_THREADS;
@@ -621,7 +684,7 @@ pycairo_paint_with_alpha (PycairoContext *o, PyObject *args) {
 }
 
 static PyObject *
-pycairo_path_extents (PycairoContext *o) {
+pycairo_path_extents (PycairoContext *o, PyObject *ignored) {
   double x1, y1, x2, y2;
   cairo_path_extents (o->ctx, &x1, &y1, &x2, &y2);
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
@@ -629,18 +692,18 @@ pycairo_path_extents (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_pop_group (PycairoContext *o) {
+pycairo_pop_group (PycairoContext *o, PyObject *ignored) {
   return PycairoPattern_FromPattern (cairo_pop_group (o->ctx), NULL);
 }
 
 static PyObject *
-pycairo_pop_group_to_source (PycairoContext *o) {
+pycairo_pop_group_to_source (PycairoContext *o, PyObject *ignored) {
   cairo_pop_group_to_source (o->ctx);
   Py_RETURN_NONE;
 }
 
 static PyObject *
-pycairo_push_group (PycairoContext *o) {
+pycairo_push_group (PycairoContext *o, PyObject *ignored) {
   cairo_push_group (o->ctx);
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
   Py_RETURN_NONE;
@@ -713,14 +776,14 @@ pycairo_rel_move_to (PycairoContext *o, PyObject *args) {
 }
 
 static PyObject *
-pycairo_reset_clip (PycairoContext *o) {
+pycairo_reset_clip (PycairoContext *o, PyObject *ignored) {
   cairo_reset_clip (o->ctx);
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
   Py_RETURN_NONE;
 }
 
 static PyObject *
-pycairo_restore (PycairoContext *o) {
+pycairo_restore (PycairoContext *o, PyObject *ignored) {
   cairo_restore (o->ctx);
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
   Py_RETURN_NONE;
@@ -739,7 +802,7 @@ pycairo_rotate (PycairoContext *o, PyObject *args) {
 }
 
 static PyObject *
-pycairo_save (PycairoContext *o) {
+pycairo_save (PycairoContext *o, PyObject *ignored) {
   cairo_save (o->ctx);
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
   Py_RETURN_NONE;
@@ -765,7 +828,7 @@ pycairo_select_font_face (PycairoContext *o, PyObject *args) {
   int slant_arg = CAIRO_FONT_SLANT_NORMAL;
   int weight_arg = CAIRO_FONT_WEIGHT_NORMAL;
 
-  if (!PyArg_ParseTuple (args, PYCAIRO_ENC_TEXT_FORMAT "|ii:Context.select_font_face",
+  if (!PyArg_ParseTuple (args, "es|ii:Context.select_font_face",
                          "utf-8", &utf8, &slant_arg, &weight_arg))
     return NULL;
 
@@ -1083,7 +1146,7 @@ pycairo_show_glyphs (PycairoContext *o, PyObject *args) {
 }
 
 static PyObject *
-pycairo_show_page (PycairoContext *o) {
+pycairo_show_page (PycairoContext *o, PyObject *ignored) {
   Py_BEGIN_ALLOW_THREADS;
   cairo_show_page (o->ctx);
   Py_END_ALLOW_THREADS;
@@ -1095,7 +1158,7 @@ static PyObject *
 pycairo_show_text (PycairoContext *o, PyObject *args) {
   const char *utf8;
 
-  if (!PyArg_ParseTuple (args, PYCAIRO_ENC_TEXT_FORMAT ":Context.show_text", "utf-8", &utf8))
+  if (!PyArg_ParseTuple (args, "es:Context.show_text", "utf-8", &utf8))
     return NULL;
 
   Py_BEGIN_ALLOW_THREADS;
@@ -1108,7 +1171,7 @@ pycairo_show_text (PycairoContext *o, PyObject *args) {
 }
 
 static PyObject *
-pycairo_stroke (PycairoContext *o) {
+pycairo_stroke (PycairoContext *o, PyObject *ignored) {
   Py_BEGIN_ALLOW_THREADS;
   cairo_stroke (o->ctx);
   Py_END_ALLOW_THREADS;
@@ -1117,7 +1180,7 @@ pycairo_stroke (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_stroke_extents (PycairoContext *o) {
+pycairo_stroke_extents (PycairoContext *o, PyObject *ignored) {
   double x1, y1, x2, y2;
   cairo_stroke_extents (o->ctx, &x1, &y1, &x2, &y2);
   RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
@@ -1125,7 +1188,7 @@ pycairo_stroke_extents (PycairoContext *o) {
 }
 
 static PyObject *
-pycairo_stroke_preserve (PycairoContext *o) {
+pycairo_stroke_preserve (PycairoContext *o, PyObject *ignored) {
   Py_BEGIN_ALLOW_THREADS;
   cairo_stroke_preserve (o->ctx);
   Py_END_ALLOW_THREADS;
@@ -1139,7 +1202,7 @@ pycairo_text_extents (PycairoContext *o, PyObject *args) {
   const char *utf8;
   PyObject *ext_args, *res;
 
-  if (!PyArg_ParseTuple (args, PYCAIRO_ENC_TEXT_FORMAT ":Context.text_extents", "utf-8", &utf8))
+  if (!PyArg_ParseTuple (args, "es:Context.text_extents", "utf-8", &utf8))
     return NULL;
 
   cairo_text_extents (o->ctx, utf8, &extents);
@@ -1157,7 +1220,7 @@ static PyObject *
 pycairo_text_path (PycairoContext *o, PyObject *args) {
   const char *utf8;
 
-  if (!PyArg_ParseTuple (args, PYCAIRO_ENC_TEXT_FORMAT ":Context.text_path", "utf-8", &utf8))
+  if (!PyArg_ParseTuple (args, "es:Context.text_path", "utf-8", &utf8))
     return NULL;
 
   cairo_text_path (o->ctx, utf8);
@@ -1229,7 +1292,7 @@ pycairo_show_text_glyphs (PycairoContext *o, PyObject *args) {
   Py_ssize_t i, clusters_size, glyphs_size;
 
   if (!PyArg_ParseTuple (args,
-      PYCAIRO_ENC_TEXT_FORMAT "OOi:Context.show_text_glyphs",
+      "esOOi:Context.show_text_glyphs",
       "utf-8", &utf8, &glyphs_arg, &clusters_arg, &cluster_flags_arg))
     return NULL;
 
@@ -1309,6 +1372,14 @@ static PyMethodDef pycairo_methods[] = {
    * - not needed since Pycairo calls Pycairo_Check_Status() to check
    *   for errors and raise exceptions
    */
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 17, 6)
+  {"set_hairline",    (PyCFunction)pycairo_set_hairline,     METH_VARARGS},
+  {"get_hairline",    (PyCFunction)pycairo_get_hairline,     METH_NOARGS},
+#endif
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 15, 10)
+  {"tag_begin",       (PyCFunction)pycairo_tag_begin,        METH_VARARGS},
+  {"tag_end",         (PyCFunction)pycairo_tag_end,          METH_VARARGS},
+#endif
   {"append_path",     (PyCFunction)pycairo_append_path,      METH_VARARGS},
   {"arc",             (PyCFunction)pycairo_arc,              METH_VARARGS},
   {"arc_negative",    (PyCFunction)pycairo_arc_negative,     METH_VARARGS},
@@ -1430,7 +1501,7 @@ pycairo_richcompare (PyObject *self, PyObject *other, int op)
   }
 }
 
-static PYCAIRO_Py_hash_t
+static Py_hash_t
 pycairo_hash (PyObject *self)
 {
   return PYCAIRO_Py_hash_t_FromVoidPtr (((PycairoContext *)self)->ctx);
